package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.entity.ObeCultureProgramRequirement;
import com.qmth.distributed.print.business.enums.CultureProgramRequirementEnum;
import com.qmth.distributed.print.business.mapper.ObeCultureProgramRequirementMapper;
import com.qmth.distributed.print.business.service.ObeCultureProgramRequirementService;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 培养方案毕业要求 服务实现类
 * </p>
 *
 * @author xf
 * @since 2024-06-05
 */
@Service
public class ObeCultureProgramRequirementServiceImpl extends ServiceImpl<ObeCultureProgramRequirementMapper, ObeCultureProgramRequirement> implements ObeCultureProgramRequirementService {

    @Override
    public List<ObeCultureProgramRequirement> listRequirement(Long cultureProgramId) {
        QueryWrapper<ObeCultureProgramRequirement> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObeCultureProgramRequirement::getCultureProgramId, cultureProgramId)
                .orderByAsc(ObeCultureProgramRequirement::getSortNum);
        List<ObeCultureProgramRequirement> obeCultureProgramRequirementList = this.list(queryWrapper);

        List<ObeCultureProgramRequirement> obeCultureProgramRequirements = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(obeCultureProgramRequirementList)) {
            Map<Long, ObeCultureProgramRequirement> obeCultureProgramRequirementMap = obeCultureProgramRequirementList.stream().filter(obeCultureProgramRequirement -> obeCultureProgramRequirement.getParentId() == null).collect(Collectors.toMap(ObeCultureProgramRequirement::getId, obeCultureProgramRequirement -> obeCultureProgramRequirement));
            for (ObeCultureProgramRequirement obeCultureProgramRequirement : obeCultureProgramRequirementList) {
                if (obeCultureProgramRequirement.getParentId() != null) {
                    obeCultureProgramRequirementMap.get(obeCultureProgramRequirement.getParentId()).getSubRequirementList().add(obeCultureProgramRequirement);
                }
            }
            obeCultureProgramRequirements = obeCultureProgramRequirementMap.values().stream().collect(Collectors.toList());
            obeCultureProgramRequirements.sort(Comparator.comparingInt(ObeCultureProgramRequirement::getSortNum));
        }
        return obeCultureProgramRequirements;
    }

    @Override
    public boolean saveRequirement(ObeCultureProgramRequirement obeCultureProgramRequirement) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        // 修改
        if (obeCultureProgramRequirement.getId() != null) {
            obeCultureProgramRequirement.updateInfo(sysUser.getId());
        }
        // 新增
        else {
            obeCultureProgramRequirement.setSortNum(this.getMaxSortNum(obeCultureProgramRequirement.getCultureProgramId(), obeCultureProgramRequirement.getParentId()) + 1);
            obeCultureProgramRequirement.insertInfo(sysUser.getId());
        }
        this.saveOrUpdate(obeCultureProgramRequirement);
        // 节点数变动后，修改矩阵数据
//        List<TPCourse> tpCourseList = tpCourseService.listCourse(tpRequirement.getProfessionalId());
//        if (CollectionUtils.isNotEmpty(tpCourseList)) {
//            for (TPCourse tpCourse : tpCourseList) {
//                tpMatrixService.updateMatrixForNodeCount(tpCourse, tpRequirement);
//            }
//        }
        return true;
    }

    @Override
    public int getMaxSortNum(Long cultureProgramId, Long parentId) {
        QueryWrapper<ObeCultureProgramRequirement> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ObeCultureProgramRequirement> lambda = queryWrapper.lambda();
        lambda.eq(ObeCultureProgramRequirement::getCultureProgramId, cultureProgramId);
        if (parentId != null) {
            lambda.eq(ObeCultureProgramRequirement::getParentId, parentId);
        }
        lambda.orderByDesc(ObeCultureProgramRequirement::getSortNum);
        lambda.last("limit 1");
        ObeCultureProgramRequirement requirement = this.getOne(queryWrapper);
        if (requirement == null) {
            return 0;
        } else {
            return requirement.getSortNum();
        }
    }

    @Transactional
    @Override
    public boolean removeRequirement(Long id) {
//        tpMatrixService.removeMatrixByCondition(null, null, id);
        return this.removeById(id);
    }

    @Transactional
    @Override
    public void initRequirement(Long cultureProgramId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        List<ObeCultureProgramRequirement> obeCultureProgramRequirementList = new ArrayList<>();
        for (CultureProgramRequirementEnum value : CultureProgramRequirementEnum.values()) {
            // 一级节点
            ObeCultureProgramRequirement obeCultureProgramRequirement = new ObeCultureProgramRequirement(cultureProgramId, value.name(), value.getName(), null, value.ordinal() + 1);
            obeCultureProgramRequirement.insertInfo(sysUser.getId());
            obeCultureProgramRequirementList.add(obeCultureProgramRequirement);
            // 每个一级节点默认创建一个二级节点
            ObeCultureProgramRequirement obeSubCultureProgramRequirement = new ObeCultureProgramRequirement(cultureProgramId, null, "指标点" + obeCultureProgramRequirement.getSortNum() + "-1", obeCultureProgramRequirement.getId(), 1);
            obeSubCultureProgramRequirement.insertInfo(sysUser.getId());
            obeCultureProgramRequirementList.add(obeSubCultureProgramRequirement);
        }

        this.saveBatch(obeCultureProgramRequirementList);
    }
}
