package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.distributed.print.business.entity.ObeCourseRequirementMatrix;
import com.qmth.distributed.print.business.entity.ObeCultureProgramRequirement;
import com.qmth.distributed.print.business.entity.ObeCultureProgramTarget;
import com.qmth.distributed.print.business.entity.ObeRequirementTargetMatrix;
import com.qmth.distributed.print.business.mapper.ObeCultureProgramTargetMapper;
import com.qmth.distributed.print.business.service.ObeCultureProgramRequirementService;
import com.qmth.distributed.print.business.service.ObeCultureProgramTargetService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.service.ObeRequirementTargetMatrixService;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 培养方案培养目标 服务实现类
 * </p>
 *
 * @author xf
 * @since 2024-06-05
 */
@Service
public class ObeCultureProgramTargetServiceImpl extends ServiceImpl<ObeCultureProgramTargetMapper, ObeCultureProgramTarget> implements ObeCultureProgramTargetService {

    @Resource
    private ObeCultureProgramRequirementService obeCultureProgramRequirementService;
    @Resource
    private ObeRequirementTargetMatrixService obeRequirementTargetMatrixService;

    @Override
    public List<ObeCultureProgramTarget> list(Long cultureProgramId) {
        QueryWrapper<ObeCultureProgramTarget> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObeCultureProgramTarget::getCultureProgramId, cultureProgramId)
                .orderByAsc(ObeCultureProgramTarget::getCreateTime);
        return this.list(queryWrapper);
    }

    @Transactional
    @Override
    public boolean saveData(ObeCultureProgramTarget obeCultureProgramTarget) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (obeCultureProgramTarget.getId() == null) {
            obeCultureProgramTarget.insertInfo(sysUser.getId());

            // 同时生成培养目标与毕业要求关系矩阵数据
            if (this.save(obeCultureProgramTarget)) {
                List<ObeCultureProgramRequirement> obeCultureProgramRequirementList = obeCultureProgramRequirementService.listRequirement(obeCultureProgramTarget.getCultureProgramId());
                List<ObeRequirementTargetMatrix> obeRequirementTargetMatrixList = obeCultureProgramRequirementList.stream().map(m -> new ObeRequirementTargetMatrix(obeCultureProgramTarget.getCultureProgramId(), m.getId(), obeCultureProgramTarget.getId())).collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(obeRequirementTargetMatrixList)) {
                    obeRequirementTargetMatrixService.saveBatch(obeRequirementTargetMatrixList);
                }
            }
            return true;
        } else {
            obeCultureProgramTarget.updateInfo(sysUser.getId());
            return this.updateById(obeCultureProgramTarget);
        }
    }

    @Override
    public boolean delete(Long id) {
        return this.removeById(id);
    }
}
